#ifndef AG_SET_BASE_H
#define AG_SET_BASE_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include <stdlib.h>
#include <assert.h>
#include "agabt.h"
#include "agref.h"
#include "agcntnr.h"

template <class Object>
class AgSetBase
: public AgAleatoryBinaryTree<Object>,
  public AgReferenceCounter
{
  int objectHash(const Object &e) const {return agABTHash(e);}
  int equivalent(const Object &x, const Object &y) const {return x == y;}
public:
  AgSetBase();
  AgSetBase(const AgSetBase<Object> &s);
  AgSetBase(const AgContainerContents<Object> &);

  AgSetBase<Object> &operator = (const AgSetBase<Object> &s) {
    return (AgSetBase<Object> &)AgAleatoryBinaryTree<Object>::operator = (s);}

  AgSetBase<Object> &operator = (const AgContainerContents<Object> &);
  virtual ~AgSetBase() {}
  Object *contains(const Object &) const;
  Object *add(const Object &object) {
    return &findOrCreate(root, object, objectHash(object))->object;
  }
  int remove(const Object &object) {
    return findAndRemove(root, object, objectHash(object));
  }
  int size() const {return AgAleatoryBinaryTree<Object>::size();}
  Object &operator [] (int x) const;
  int operator >= (const AgSetBase<Object> &) const;
  int operator <= (const AgSetBase<Object> &x) const {return x >= *this;}
  int operator == (const AgSetBase<Object> &x) const {return *this >= x && x >= *this;}
  int operator != (const AgSetBase<Object> &x) const {return !(*this == x);}
  int operator < (const AgSetBase<Object> &) const;
  int operator > (const AgSetBase<Object> &x) const {return x < *this;}
};

template <class Object>
AgSetBase<Object>::AgSetBase()
: AgAleatoryBinaryTree<Object>(),
  AgReferenceCounter()
{
  // Nothing to do
}

template <class Object>
AgSetBase<Object>::AgSetBase(const AgSetBase<Object> &s)
: AgAleatoryBinaryTree<Object>(s),
  AgReferenceCounter()
{
  // Nothing to do
}

template <class Object>
AgSetBase<Object>::AgSetBase(const AgContainerContents<Object> &c)
: AgAleatoryBinaryTree<Object>(),
  AgReferenceCounter()
{
  int i = 0;
  while (i < c.size()) add(c[i++]);
}

template <class Object>
AgSetBase<Object> &AgSetBase<Object>::operator = (const AgContainerContents<Object> &c)  {
  reset();
  int i = 0;
  while (i < c.size()) add(c[i++]);
  return *this;
}

// Determines whether a set contains a specified object
template <class Object>
Object *AgSetBase<Object>::contains(const Object &object) const {
  AgAleatoryBinaryNode<Object> *&node = find(root, object, objectHash(object));
  return node ? &node->object : NULL;
}

// Identifies the k'th object in the set
template <class Object>
Object &AgSetBase<Object>::operator [] (int x) const {
  assert((unsigned) x < (unsigned) size());
  return root->find(x);
}

template <class Object>
int AgSetBase<Object>::operator >= (const AgSetBase<Object> &x) const {
  if (this == &x) return 1;
  int n = x.size();
  if (n > size()) return 0;
  while (n--) if (!contains(x[n])) return 0;
  return 1;
}

template <class Object>
int AgSetBase<Object>::operator < (const AgSetBase<Object> &x) const {
  if (!(*this <= x)) return 0;
  int n = size();
  while (n--) if (!contains(x[n])) return 1;
  return 0;
}

#endif
