#ifndef AG_REFERENCE_H
#define AG_REFERENCE_H

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

#include <stddef.h>

class AgReferenceCounter {
  int useCount;
public:
  AgReferenceCounter() : useCount(1) {}
  AgReferenceCounter(const AgReferenceCounter &) : useCount(1) {}
  void lockStorage() {useCount++;}
  int unlockStorage() {return --useCount == 0;}
  int isUniqueInstance() {return useCount == 1;}
  int isNotUniqueInstance() {return useCount != 1;}
};

template <class ContainerBase>
class AgCopyControl {
protected:
  ContainerBase *base;
  AgCopyControl(ContainerBase *x) : base(x) {}
  AgCopyControl<ContainerBase> &operator =(const AgCopyControl<ContainerBase> &x);
  void ensureUniqueInstance();
public:
  AgCopyControl(const AgCopyControl<ContainerBase> &x) : base(x.base) {base->lockStorage();}
  ~AgCopyControl();
  const ContainerBase *kernel() const{return base;}
  ContainerBase *&kernel() {ensureUniqueInstance(); return base;}
  int isUniqueInstance() {return base->isUniqueInstance();}
  int isNotUniqueInstance() {return base->isNotUniqueInstance();}
};

template <class ContainerBase>
AgCopyControl<ContainerBase>::~AgCopyControl() {
  if (base->unlockStorage()) delete base;
}

template <class ContainerBase>
AgCopyControl<ContainerBase> &AgCopyControl<ContainerBase>::operator =(const AgCopyControl<ContainerBase> &x) {
  if (&x == this) return *this;
  if (base->unlockStorage()) delete base;
  base = x.base;
  base->lockStorage();
  return *this;
}

template <class ContainerBase>
void AgCopyControl<ContainerBase>::ensureUniqueInstance() {
  if (base->isUniqueInstance()) return;
  base->unlockStorage();
  base = new ContainerBase(*base);
}

template <class ContainerBase>
inline int operator == (const AgCopyControl<ContainerBase> &x, const AgCopyControl<ContainerBase> &y) {
  return *x.kernel() == *y.kernel();
}

template <class ContainerBase>
inline int operator != (const AgCopyControl<ContainerBase> &x, const AgCopyControl<ContainerBase> &y) {
  return !(x == y);
}

#endif
