#include "agbstr.h"
#include <string.h>

/*
 * The AGCLIB1 Class Library, Version 1.0
 * Copyright (c) Parsifal Software, 2001. All Rights Reserved.
 * http://www.parsifalsoft.com
*/

AgStringBase::AgStringBase()
  : AgReferenceCounter(),
    bufferSize(16),
    buffer(new char[1+bufferSize]),
    count(0)
{
  buffer[0] = 0;
}

AgStringBase::AgStringBase(int n)
  : AgReferenceCounter(),
    bufferSize(n),
    buffer(new char[1+bufferSize]),
    count(0)
{
  buffer[0] = 0;
}

AgStringBase::AgStringBase(const char *s)
  : AgReferenceCounter(),
    bufferSize(s ? strlen(s) : 16),
    buffer(new char[1+bufferSize]),
    count(s ? strlen(s) : 0)
{
  if (s) strcpy(buffer, s); else buffer[0] = 0;
}

AgStringBase::AgStringBase(const AgStringBase &b)
  : AgReferenceCounter(),
    bufferSize(b.bufferSize),
    buffer(new char[1+bufferSize]),
    count(b.count)
{
  memcpy(buffer, b.buffer, count+1);
}

AgStringBase &AgStringBase::operator = (const AgStringBase &b) {
  if (&b == this) return *this;
  delete buffer;
  bufferSize = b.bufferSize;
  buffer = new char[1+bufferSize];
  count = b.count;
  memcpy(buffer, b.buffer, count+1);
  return *this;
}

AgStringBase &AgStringBase::append(int c) {
  if (count >= bufferSize) {
    if (bufferSize) bufferSize *= 2;
    else bufferSize = 16;
    char *newBuffer = new char[1+bufferSize];
    memcpy(newBuffer, buffer, count);
    delete buffer;
    buffer = newBuffer;
  }
  buffer[count++] = (char) c;
  buffer[count] = 0;
  return *this;
}

AgStringBase &AgStringBase::append(const char *s) {
  if (s == NULL) return *this;
  int newcount = count + strlen(s);
  if (newcount == 0) return *this;
  if (newcount >= bufferSize) {
    for (bufferSize = 1; newcount >= bufferSize; bufferSize *= 2) {}
    char *newBuffer = new char[1+bufferSize];
    memcpy(newBuffer, buffer, count+1);
    delete buffer;
    buffer = newBuffer;
  }
  strcat(buffer,s);
  count = newcount;
  return *this;
}

AgStringBase &AgStringBase::append(const char *s, int n) {
  int newcount = count + n;
  if (s == NULL || newcount == 0) return *this;
  if (newcount >= bufferSize) {
    for (bufferSize = 1; newcount >= bufferSize; bufferSize *= 2) {}
    char *newBuffer = new char[1+bufferSize];
    memcpy(newBuffer, buffer, count+1);
    delete buffer;
    buffer = newBuffer;
  }
  strncat(buffer,s,n);
  count = newcount;
  buffer[count] = 0;
  return *this;
}

int agABTHash(const char *s, int hashcode) {
  while (*s) hashcode = (hashcode + *s++ & 0xff)*257 >> 4;
  return hashcode;
}

