#ifndef AST_H_1027348850
#define AST_H_1027348850

#ifndef AG_OBJECT_WRAPPER_DEFINED
#define AG_OBJECT_WRAPPER_DEFINED

#ifndef AG_PLACEMENT_DELETE_REQUIRED
#if _MSC_VER >= 1200 || __INTEL_COMPILER
#define AG_PLACEMENT_DELETE_REQUIRED 1
#endif
#endif

template <class Object>
class AgObjectWrapper {
  Object object;
public:
  void operator delete(void *) {}
  void *operator new(size_t, void *p) { return p;}
#if AG_PLACEMENT_DELETE_REQUIRED
  void operator delete(void *, void *) { }
#endif
  AgObjectWrapper(const Object &o) : object(o) {}
  ~AgObjectWrapper() {}
  operator Object &() {return object;}
};

#endif

typedef union {
  double alignment;
  char ag_vt_2[sizeof(int)];
  char ag_vt_3[sizeof(long)];
  char ag_vt_4[sizeof(AgObjectWrapper<AgString >)];
  char ag_vt_5[sizeof(AgObjectWrapper<Value >)];
  char ag_vt_6[sizeof(AgObjectWrapper<AgStack<AstNode *> >)];
  char ag_vt_7[sizeof(AgObjectWrapper<AgStack<AgString> >)];
  char ag_vt_8[sizeof(AstNode *)];
  char ag_vt_9[sizeof(Opcode)];
  char ag_vt_10[sizeof(double)];
} parse_vs_type;

typedef enum {
  parse_white_space_token = 1, parse_script_token = 7,
  parse_local_statement_token, parse_statement_list_token, parse_eof_token,
  parse_statement_token, parse_open_statement_token,
  parse_closed_statement_token, parse_if_condition_token,
  parse_WHILE_token = 16, parse_expression_token, parse_FOR_token = 19,
  parse_lvalue_token, parse_for_increment_token = 22,
  parse_simple_statement_token = 24, parse_REPEAT_token = 26,
  parse_UNTIL_token, parse_compound_statement_token = 29,
  parse_optional_expression_token = 31, parse_dump_statement_token,
  parse_print_statement_token, parse_function_definition_token,
  parse_name_list_token = 36, parse_function_body_token = 51,
  parse_simple_expression_token, parse_relational_op_token,
  parse_term_token, parse_additive_op_token, parse_unary_expression_token,
  parse_multiplicative_op_token, parse_factor_token,
  parse_unary_op_token = 60, parse_primary_token, parse_constant_token = 63,
  parse_function_call_token, parse_optional_arg_list_token = 67,
  parse_arg_list_token, parse_string_token, parse_space_token = 87,
  parse_letter_token = 98, parse_simple_real_token = 100,
  parse_signed_exponent_token = 102, parse_integer_part_token,
  parse_fraction_part_token = 105, parse_decimal_integer_token,
  parse_hybrid_integer_token, parse_octal_integer_token,
  parse_exponent_token = 110, parse_digit_token, parse_hex_integer_token,
  parse_hex_digit_token = 119, parse_s_char_sequence_token = 122,
  parse_s_char_token, parse_escape_sequence_token = 125,
  parse_simple_escape_sequence_token, parse_octal_escape_sequence_token,
  parse_hexadecimal_escape_sequence_token, parse_one_octal_token = 140,
  parse_two_octal_token, parse_three_octal_token,
  parse_hexadecimal_digit_token = 145, parse_c_char_token = 147,
  parse_name_token = 172, parse_integer_token = 183, parse_real_token,
  parse_character_constant_token, parse_string_element_token = 203
} parse_token_type;

typedef struct parse_pcb_struct{
  parse_token_type token_number, reduction_token, error_frame_token;
  int input_code;
  int input_value;
  int line, column;
  int ssx, sn, error_frame_ssx;
  int drt, dssx, dsn;
  int ss[128];
  parse_vs_type vs[128];
  int ag_ap;
  char *error_message;
  char read_flag;
  char exit_flag;
  FileLocation input_context;
  FileLocation cs[128];
  int bts[128], btsx;
  const unsigned char * pointer;
  const unsigned char * la_ptr;
  const unsigned char *key_sp;
  int save_index, key_state;
  char ag_msg[82];
/*  Line 128, W:/agdev/agex/packages/xidek/pll/function/ast.syn */
    // Declare an abstract syntax tree
    Tree ast;

    int loopDepth;
    // Constructor for pcb
    parse_pcb_struct(const char *text);

    // Functions used during parsing
    void checkLoop();
    void reportError();
  } parse_pcb_type;

#ifndef PRULE_CONTEXT
#define PRULE_CONTEXT(pcb)  (&((pcb).cs[(pcb).ssx]))
#define PERROR_CONTEXT(pcb) ((pcb).cs[(pcb).error_frame_ssx])
#define PCONTEXT(pcb)       ((pcb).cs[(pcb).ssx])
#endif

#ifndef AG_RUNNING_CODE_CODE
/* PCB.exit_flag values */
#define AG_RUNNING_CODE         0
#define AG_SUCCESS_CODE         1
#define AG_SYNTAX_ERROR_CODE    2
#define AG_REDUCTION_ERROR_CODE 3
#define AG_STACK_ERROR_CODE     4
#define AG_SEMANTIC_ERROR_CODE  5
#endif
void init_parse(parse_pcb_type *);
void parse(parse_pcb_type *);
#endif

